#include <stdio.h>
#include "mex.h" 
#include "mylw.h" 
/* mylw.h contains the model-specific implementation of lw 
   and is generated by compile_lw.m */

#define MAX(X,Y) (X)>(Y)?(X):(Y)

int sample_discrete(int n, const double p[], double cum[], double R)
{
  int i;
  /* printf("sample_discrete R=%f\n", R);
  for (i = 0; i < n; i++) {
    printf("p[%d] = %f\n", i, p[i]);
  }
  */

  cum[0] = p[0];
  for (i=1; i < n; i++) {
    cum[i] = cum[i-1] + p[i];
    /* printf("cum[%d]=%f\n", i, cum[i]); */
  }
  for (i = 0; i < n; i++) {
    if (R < cum[i])
      return i;
  }
  printf("error in sample_discrete\n");
  return n;
}

void matlab_rand(int nrows, int ncols, double *rnd)
{
  /* Get some random numbers from matlab.
     We assume rnd has been pre-allocated to store nrows*ncols doubles */

  mxArray *input_array[2], *output_array[1];
  double *args, *matlab_rnd_array, temp;
  int i, r, c;

  input_array[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[0]);
  *args = nrows;

  input_array[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[1]);
  *args = ncols;
  
  mexCallMATLAB(1, output_array, 2, input_array, "rand");
  matlab_rnd_array = mxGetPr(output_array[0]);

  /* copy results from matlab's workspace to C's */
  for (r = 0; r < nrows; r++) {
    for (c = 0; c < ncols; c++) {
      temp = matlab_rnd_array[r+nrows*c];
      rnd[r+nrows*c] = temp;
      /* printf("R[%d,%d]=%f\n", r, c, temp); */
    }
  }

}

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
                 )
{
  double *ev, *obs, *samples;
  double *weights, *rnd;
  int nsamples, nnodes, i, max_node;
  int *int_ev, *int_obs;

  /* read the input args */
  nsamples = mxGetScalar(prhs[0]);
  ev = mxGetPr(prhs[1]); nnodes = MAX(mxGetN(prhs[1]), mxGetM(prhs[1]));
  obs = mxGetPr(prhs[2]);
  max_node = mxGetScalar(prhs[3]);


  /* set the output pointers  */
  plhs[0] = mxCreateDoubleMatrix(nsamples, nnodes, mxREAL);
  samples = mxGetPr(plhs[0]);
  
  plhs[1] = mxCreateDoubleMatrix(1, nsamples, mxREAL);
  weights = mxGetPr(plhs[1]);
  
  
  /* do work */

  rnd = malloc(nsamples * nnodes * sizeof(double));
  matlab_rand(nnodes, nsamples, rnd);  

  /* mylw((int*) ev, (int*) obs, nsamples, samples, weights, rnd); */
  int_ev = malloc(nnodes * sizeof(int));
  int_obs = malloc(nnodes * sizeof(int));
  for (i=0; i < nnodes; i++) {
    int_ev[i] = (int) ev[i];
    int_obs[i] = (int) obs[i];
  }
  mylw(int_ev, int_obs, nsamples, samples, weights, rnd, max_node);
  free(int_ev);
  free(int_obs);
  free(rnd);
}

