function bnet = mk_zero_mean(bnet)
% MK_ZERO_MEAN Make all the nodes except the hidden nodes in the first slice have permanently zero mean.
% bnet = mk_zero_mean(bnet)
%
% In a Linear Dynamical System, we assume the Gaussian noise is zero mean for all nodes.
% However, we need a way of specifying the initial mean m and variance V of X(1).
% We can do this by making X(1) have noise N(m,V).

num_equiv_classes = max(bnet.equiv_class(:));
for i=1:num_equiv_classes
  if ismember(i, bnet.hidden_slice) & (i <= bnet.slice_size)
    bnet.bnodes{i}.adjustable_mean = 1;
  else
    bnet.bnodes{i}.mean = zeros(bnet.node_sizes(i),1);
    bnet.bnodes{i}.adjustable_mean = 0;
  end
end

