#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mex.h"

/*
MEX utility function called by fast_setdiff, fast_intersect, fast_subset.
The output will be sorted iff the inputs are.
We do not assume the inputs are sorted.
*/


#define SETDIFF 0
#define INTERSECT 1
#define SUBSET 2

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  int i, j, na, nb, num_inb;
  double *A, *B, *C;
  int fn;
  int *inb;

  A = mxGetPr(prhs[0]);
  na = length(prhs[0]);

  B = mxGetPr(prhs[1]);
  nb = length(prhs[1]);

  fn = mxGetScalar(prhs[2]);

  /* Find out, for each element of A, whether it is in B or not */
  inb = (int*) malloc(na*sizeof(int));
  num_inb = 0;

  for (i = 0; i < na; i++) {
    inb[i] = 0;
    for (j = 0; j < nb; j++) {
      if (A[i]==B[j]) {
	inb[i] = 1;
	num_inb++;
      }
    }
  }


  switch (fn) {
  case SETDIFF:
    plhs[0] = mxCreateDoubleMatrix(1, na-num_inb, mxREAL);
    C = mxGetPr(plhs[0]);
    j = 0;
    for (i = 0; i < na; i++) {
      if (!inb[i]) {
	C[j] = A[i];
	j++;
      }
    }
    break;

  case INTERSECT:
    plhs[0] = mxCreateDoubleMatrix(1, num_inb, mxREAL);
    C = mxGetPr(plhs[0]);
    j = 0;
    for (i = 0; i < na; i++) {
      if (inb[i]) {
	C[j] = A[i];
	j++;
      }
    }
    break;
    
  case SUBSET:
    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    C = mxGetPr(plhs[0]);
    *C = (num_inb == na) ? 1 : 0;
    break;
  }

  free(inb);
}


int length(const mxArray *p)
{
  int r, c, n;
  r = mxGetM(p);
  c = mxGetN(p);
  n = (r > c) ? r : c;
  if ((r==0) || (c==0)) n = 0; /* empty matrix */
  return n;
}




