#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mex.h"

/*
MEX version of draw_from_discrete_distribution.
*/

void matlab_rand(int r, int c, double *R)
{
  /* get some random numbers from matlab */

  mxArray *input_array[2], *output_array[1];
  double *args;

  input_array[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[0]);
  *args = r;

  input_array[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[1]);
  *args = c;
  
  mexCallMATLAB(1, output_array, 2, input_array, "rand");
  R = mxGetPr(output_array[0]);
}


void draw(const double *prob, int n, int r, int c, double *M)
{
  int i, j, k, count;
  mxArray *input_array[2], *output_array[1];
  double *R, *args;
  double *cumprob;
  
  cumprob = calloc(n+1, sizeof(double)); /* vector of length n+1 */
  if (cumprob == NULL) {
    mexErrMsgTxt("could not allocate space for cumprob\n");
  }
  cumprob[0] = 0;
  for (i = 1; i <= n; i++) {
    cumprob[i] = cumprob[i-1] + prob[i-1];
  }
  if (cumprob[n] < 1-1e-3) {
    /*    fprintf(stderr, "distribution does not sum to 1\n")
          exit(0); */
    mexErrMsgTxt("distribution does not sum to 1\n");
  } 

  /*
  printf("cumprob\n");
  for (i = 0; i <= n; i++)
    printf("%f ", cumprob[i]);
  printf("\n");
  */

  /*
  printf("prob\n");
  for (i = 0; i < n; i++)
    printf("%f ", prob[i]);
  printf("\n");
  */

  /* get some random numbers from matlab */
  input_array[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[0]);
  *args = r;

  input_array[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
  args = mxGetPr(input_array[1]);
  *args = c;
  
  mexCallMATLAB(1, output_array, 2, input_array, "rand");
  R = mxGetPr(output_array[0]);


  /*   matlab_rand(r,c,R); */

#if 0
  printf("R\n");
  count = 0;
  for (i = 0; i < r; i++) {
    for (j = 0; j < c; j++) {
      printf("%f ", R[count]);
      count++;
    }
    printf("\n");
  }
#endif

  count = 0;
  for (i = 0; i < r; i++) {
    for (j = 0; j < c; j++) {
      /*      *(M+count) = 0; */
      M[count] = 1;
      count++;
    }
  }

  for (k = 0; k < n; k++) {
    count = 0;
    for (i = 0; i < r; i++) {
      for (j = 0; j < c; j++) {
	if ((R[count] >= cumprob[k]) && (R[count] < cumprob[k+1])) {
	  M[count] = k+1;
	}
	count++;
      }
    }
  }

  free(cumprob);
}


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  double *prob;
  int r, c, n, m;
  double *M;

  /* read the input args */
  prob = mxGetPr(prhs[0]);
  r = mxGetScalar(prhs[1]);
  c = mxGetScalar(prhs[2]);

  /* get size of prob */
  n = mxGetN(prhs[0]);
  m = mxGetM(prhs[0]);
  if (m > n) n=m;

  /* set the output pointer */
  plhs[0] = mxCreateDoubleMatrix(r, c, mxREAL);
  M = mxGetPr(plhs[0]);

  draw(prob, n, r, c, M);
}


